# coding: utf-8

"""
Influx API Service.

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

OpenAPI spec version: 0.1.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class PermissionResource(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'type': 'str',
        'id': 'str',
        'name': 'str',
        'org_id': 'str',
        'org': 'str'
    }

    attribute_map = {
        'type': 'type',
        'id': 'id',
        'name': 'name',
        'org_id': 'orgID',
        'org': 'org'
    }

    def __init__(self, type=None, id=None, name=None, org_id=None, org=None):  # noqa: E501,D401,D403
        """PermissionResource - a model defined in OpenAPI."""  # noqa: E501
        self._type = None
        self._id = None
        self._name = None
        self._org_id = None
        self._org = None
        self.discriminator = None

        self.type = type
        self.id = id
        self.name = name
        self.org_id = org_id
        self.org = org

    @property
    def type(self):
        """Get the type of this PermissionResource.

        :return: The type of this PermissionResource.
        :rtype: str
        """  # noqa: E501
        return self._type

    @type.setter
    def type(self, type):
        """Set the type of this PermissionResource.

        :param type: The type of this PermissionResource.
        :type: str
        """  # noqa: E501
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501
        self._type = type

    @property
    def id(self):
        """Get the id of this PermissionResource.

        If ID is set that is a permission for a specific resource. if it is not set it is a permission for all resources of that resource type.

        :return: The id of this PermissionResource.
        :rtype: str
        """  # noqa: E501
        return self._id

    @id.setter
    def id(self, id):
        """Set the id of this PermissionResource.

        If ID is set that is a permission for a specific resource. if it is not set it is a permission for all resources of that resource type.

        :param id: The id of this PermissionResource.
        :type: str
        """  # noqa: E501
        self._id = id

    @property
    def name(self):
        """Get the name of this PermissionResource.

        Optional name of the resource if the resource has a name field.

        :return: The name of this PermissionResource.
        :rtype: str
        """  # noqa: E501
        return self._name

    @name.setter
    def name(self, name):
        """Set the name of this PermissionResource.

        Optional name of the resource if the resource has a name field.

        :param name: The name of this PermissionResource.
        :type: str
        """  # noqa: E501
        self._name = name

    @property
    def org_id(self):
        """Get the org_id of this PermissionResource.

        If orgID is set that is a permission for all resources owned my that org. if it is not set it is a permission for all resources of that resource type.

        :return: The org_id of this PermissionResource.
        :rtype: str
        """  # noqa: E501
        return self._org_id

    @org_id.setter
    def org_id(self, org_id):
        """Set the org_id of this PermissionResource.

        If orgID is set that is a permission for all resources owned my that org. if it is not set it is a permission for all resources of that resource type.

        :param org_id: The org_id of this PermissionResource.
        :type: str
        """  # noqa: E501
        self._org_id = org_id

    @property
    def org(self):
        """Get the org of this PermissionResource.

        Optional name of the organization of the organization with orgID.

        :return: The org of this PermissionResource.
        :rtype: str
        """  # noqa: E501
        return self._org

    @org.setter
    def org(self, org):
        """Set the org of this PermissionResource.

        Optional name of the organization of the organization with orgID.

        :param org: The org of this PermissionResource.
        :type: str
        """  # noqa: E501
        self._org = org

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, PermissionResource):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other



"""Utils to get right Date parsing function."""

from dateutil import parser

date_helper = None


class DateHelper:
    """DateHelper to groups different implementations of date operations."""

    def parse_date(self, date_string: str):
        """
        Parse string into Date or Timestamp.

        :return: Returns a :class:`datetime.datetime` object or compliant implementation
                 like :class:`class 'pandas._libs.tslibs.timestamps.Timestamp`
        """
        pass

    def to_nanoseconds(self, delta):
        """
        Get number of nanoseconds in timedelta.

        Solution comes from v1 client. Thx.
        https://github.com/influxdata/influxdb-python/pull/811
        """
        nanoseconds_in_days = delta.days * 86400 * 10 ** 9
        nanoseconds_in_seconds = delta.seconds * 10 ** 9
        nanoseconds_in_micros = delta.microseconds * 10 ** 3

        return nanoseconds_in_days + nanoseconds_in_seconds + nanoseconds_in_micros


def get_date_helper() -> DateHelper:
    """
    Return DateHelper with proper implementation.

    If there is a 'ciso8601' than use 'ciso8601.parse_datetime' else use 'dateutil.parse'.
    """
    global date_helper
    if date_helper is None:
        date_helper = DateHelper()
        try:
            import ciso8601
            date_helper.parse_date = ciso8601.parse_datetime
        except ModuleNotFoundError:
            date_helper.parse_date = parser.parse

    return date_helper



"""
A bucket is a named location where time series data is stored.

All buckets have a retention policy, a duration of time that each data point persists.
A bucket belongs to an organization.
"""


from influxdb_client import BucketsService, Bucket, PostBucketRequest


class BucketsApi(object):
    """Implementation for '/api/v2/buckets' endpoint."""

    def __init__(self, influxdb_client):
        """Initialize defaults."""
        self._influxdb_client = influxdb_client
        self._buckets_service = BucketsService(influxdb_client.api_client)

    def create_bucket(self, bucket=None, bucket_name=None, org_id=None, retention_rules=None,
                      description=None) -> Bucket:
        """Create a bucket.

        :param Bucket bucket: bucket to create (required)
        :param bucket_name: bucket name
        :param description: bucket description
        :param org_id: org_id
        :param bucket_name: bucket name
        :param retention_rules: retention rules array or single BucketRetentionRules
        :return: Bucket
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if retention_rules is None:
            retention_rules = []

        rules = []

        if isinstance(retention_rules, list):
            rules.extend(retention_rules)
        else:
            rules.append(retention_rules)

        if bucket is None:

            bucket = PostBucketRequest(name=bucket_name, retention_rules=rules, description=description)

            if org_id is None:
                org_id = self._influxdb_client.org
            bucket.org_id = org_id

        return self._buckets_service.post_buckets(post_bucket_request=bucket)

    def delete_bucket(self, bucket):
        """Delete a bucket.

        :param bucket: bucket id or Bucket
        :return: Bucket
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if isinstance(bucket, Bucket):
            bucket_id = bucket.id
        else:
            bucket_id = bucket

        return self._buckets_service.delete_buckets_id(bucket_id=bucket_id)

    def find_bucket_by_id(self, id):
        """Find bucket by ID.

        :param id:
        :return:
        """
        return self._buckets_service.get_buckets_id(id)

    def find_bucket_by_name(self, bucket_name):
        """Find bucket by name.

        :param bucket_name: bucket name
        :return: Bucket
        """
        buckets = self._buckets_service.get_buckets(name=bucket_name)

        if len(buckets.buckets) > 0:
            return buckets.buckets[0]
        else:
            return None

    def find_buckets(self):
        """Get all buckets."""
        return self._buckets_service.get_buckets()



"""Implementation for Retry strategy during HTTP requests."""

import logging
from itertools import takewhile
from random import random

from urllib3 import Retry

from influxdb_client.client.exceptions import InfluxDBError

logger = logging.getLogger(__name__)


class WritesRetry(Retry):
    """
    Writes retry configuration.

    :param int jitter_interval: random milliseconds when retrying writes
    :param int max_retry_delay: maximum delay when retrying write
    :param int exponential_base: base for the exponential retry delay, the next delay is computed as
                                 `backoff_factor * exponential_base^(attempts-1) + random(jitter_interval)`
    """

    def __init__(self, jitter_interval=0, max_retry_delay=180, exponential_base=5, **kw):
        """Initialize defaults."""
        super().__init__(**kw)
        self.jitter_interval = jitter_interval
        self.max_retry_delay = max_retry_delay
        self.exponential_base = exponential_base

    def new(self, **kw):
        """Initialize defaults."""
        if 'jitter_interval' not in kw:
            kw['jitter_interval'] = self.jitter_interval
        if 'max_retry_delay' not in kw:
            kw['max_retry_delay'] = self.max_retry_delay
        if 'exponential_base' not in kw:
            kw['exponential_base'] = self.exponential_base
        return super().new(**kw)

    def is_retry(self, method, status_code, has_retry_after=False):
        """is_retry doesn't require retry_after header. If there is not Retry-After we will use backoff."""
        if not self._is_method_retryable(method):
            return False

        return self.total and (status_code >= 429)

    def get_backoff_time(self):
        """Variant of exponential backoff with initial and max delay and a random jitter delay."""
        # We want to consider only the last consecutive errors sequence (Ignore redirects).
        consecutive_errors_len = len(
            list(
                takewhile(lambda x: x.redirect_location is None, reversed(self.history))
            )
        )
        # First fail doesn't increase backoff
        consecutive_errors_len -= 1
        if consecutive_errors_len < 0:
            return 0

        backoff_value = self.backoff_factor * (self.exponential_base ** consecutive_errors_len) + self._jitter_delay()
        return min(self.max_retry_delay, backoff_value)

    def get_retry_after(self, response):
        """Get the value of Retry-After header and append random jitter delay."""
        retry_after = super().get_retry_after(response)
        if retry_after:
            retry_after += self._jitter_delay()
        return retry_after

    def increment(self, method=None, url=None, response=None, error=None, _pool=None, _stacktrace=None):
        """Return a new Retry object with incremented retry counters."""
        new_retry = super().increment(method, url, response, error, _pool, _stacktrace)

        if response is not None:
            parsed_error = InfluxDBError(response=response)
        elif error is not None:
            parsed_error = error
        else:
            parsed_error = f"Failed request to: {url}"

        message = f"The retriable error occurred during request. Reason: '{parsed_error}'."
        if isinstance(parsed_error, InfluxDBError):
            message += f" Retry in {parsed_error.retry_after}s."

        logger.warning(message)

        return new_retry

    def _jitter_delay(self):
        return self.jitter_interval * random()



# coding: utf-8

"""
Influx API Service.

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

OpenAPI spec version: 0.1.0
Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class OnboardingRequest(object):
    """NOTE: This class is auto generated by OpenAPI Generator.

    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'username': 'str',
        'password': 'str',
        'org': 'str',
        'bucket': 'str',
        'retention_period_hrs': 'int'
    }

    attribute_map = {
        'username': 'username',
        'password': 'password',
        'org': 'org',
        'bucket': 'bucket',
        'retention_period_hrs': 'retentionPeriodHrs'
    }

    def __init__(self, username=None, password=None, org=None, bucket=None, retention_period_hrs=None):  # noqa: E501,D401,D403
        """OnboardingRequest - a model defined in OpenAPI."""  # noqa: E501
        self._username = None
        self._password = None
        self._org = None
        self._bucket = None
        self._retention_period_hrs = None
        self.discriminator = None

        self.username = username
        self.password = password
        self.org = org
        self.bucket = bucket
        if retention_period_hrs is not None:
            self.retention_period_hrs = retention_period_hrs

    @property
    def username(self):
        """Get the username of this OnboardingRequest.

        :return: The username of this OnboardingRequest.
        :rtype: str
        """  # noqa: E501
        return self._username

    @username.setter
    def username(self, username):
        """Set the username of this OnboardingRequest.

        :param username: The username of this OnboardingRequest.
        :type: str
        """  # noqa: E501
        if username is None:
            raise ValueError("Invalid value for `username`, must not be `None`")  # noqa: E501
        self._username = username

    @property
    def password(self):
        """Get the password of this OnboardingRequest.

        :return: The password of this OnboardingRequest.
        :rtype: str
        """  # noqa: E501
        return self._password

    @password.setter
    def password(self, password):
        """Set the password of this OnboardingRequest.

        :param password: The password of this OnboardingRequest.
        :type: str
        """  # noqa: E501
        if password is None:
            raise ValueError("Invalid value for `password`, must not be `None`")  # noqa: E501
        self._password = password

    @property
    def org(self):
        """Get the org of this OnboardingRequest.

        :return: The org of this OnboardingRequest.
        :rtype: str
        """  # noqa: E501
        return self._org

    @org.setter
    def org(self, org):
        """Set the org of this OnboardingRequest.

        :param org: The org of this OnboardingRequest.
        :type: str
        """  # noqa: E501
        if org is None:
            raise ValueError("Invalid value for `org`, must not be `None`")  # noqa: E501
        self._org = org

    @property
    def bucket(self):
        """Get the bucket of this OnboardingRequest.

        :return: The bucket of this OnboardingRequest.
        :rtype: str
        """  # noqa: E501
        return self._bucket

    @bucket.setter
    def bucket(self, bucket):
        """Set the bucket of this OnboardingRequest.

        :param bucket: The bucket of this OnboardingRequest.
        :type: str
        """  # noqa: E501
        if bucket is None:
            raise ValueError("Invalid value for `bucket`, must not be `None`")  # noqa: E501
        self._bucket = bucket

    @property
    def retention_period_hrs(self):
        """Get the retention_period_hrs of this OnboardingRequest.

        :return: The retention_period_hrs of this OnboardingRequest.
        :rtype: int
        """  # noqa: E501
        return self._retention_period_hrs

    @retention_period_hrs.setter
    def retention_period_hrs(self, retention_period_hrs):
        """Set the retention_period_hrs of this OnboardingRequest.

        :param retention_period_hrs: The retention_period_hrs of this OnboardingRequest.
        :type: int
        """  # noqa: E501
        self._retention_period_hrs = retention_period_hrs

    def to_dict(self):
        """Return the model properties as a dict."""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Return the string representation of the model."""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`."""
        return self.to_str()

    def __eq__(self, other):
        """Return true if both objects are equal."""
        if not isinstance(other, OnboardingRequest):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Return true if both objects are not equal."""
        return not self == other
